#!/usr/bin/env python
import configparser
import os
import textwrap
import pandas as pd
from pathlib import Path

from phylofisher import help_formatter, tools


def parse_user_inc_exc(input_file):
    """

    :return:
    """
    with open(input_file, 'r') as infile:
        user_set = set()
        for line in infile:
            line = line.strip()
            user_set.add(line)

        return user_set


def make_subset_tsv():
    """

    :return:
    """
    taxa = tools.parse_metadata(metadata)
    df = taxa_comp.to_frame()
    df = df.rename(columns={0: 'Completeness'})

    if args.chimeras:
        with open(args.chimeras, 'r') as infile:
            for line in infile:
                line = line.strip()
                split_line = line.split('\t')
                taxa[split_line[0]] = split_line[1:3]

    # Add Taxonomic Groups to DataFrame
    high_tax_list = [taxa[ind][0] for ind in df.index]
    df['Higher Taxonomy'] = high_tax_list
    low_tax_list = [taxa[ind][1] for ind in df.index]
    df['Lower Taxonomy'] = low_tax_list

    # Reorder DataFrame putting completeness in last column and sort
    cols = df.columns.tolist()
    cols = cols[1:] + cols[:1]
    df['Completeness'] = df['Completeness'] * 100
    df['Completeness'] = df['Completeness'].round(2)
    df = df[cols]
    df = df.sort_values(by=['Higher Taxonomy', 'Lower Taxonomy', 'Completeness'])

    # Add "Include in Subset" column to DataFrame and pre-fill out
    to_keep = ['yes' for k in df.index]
    df['Include in Subset'] = to_keep

    if args.to_include:
        include_set = parse_user_inc_exc(args.to_include)
    else:
        include_set = set()

    if args.to_exclude:
        exclude_set = parse_user_inc_exc(args.to_exclude)
    else:
        exclude_set = set()

    for taxon, row in df.iterrows():
        if (row['Lower Taxonomy'] in exclude_set) or (row['Higher Taxonomy'] in exclude_set) or (taxon in exclude_set):
            df.at[taxon, 'Include in Subset'] = 'no'

        if (row['Lower Taxonomy'] in include_set) or (row['Higher Taxonomy'] in include_set) or (taxon in include_set):
            df.at[taxon, 'Include in Subset'] = 'yes'

    df.to_csv(f'select_taxa.tsv', sep='\t')


def update_df_ortho(df):
    """

    :return:
    """
    with open('select_orthologs.tsv', 'r') as infile:
        infile.readline()
        to_drop = []
        for line in infile:
            line = line.strip()
            gene, _, include = line.split()
            if include == 'no':
                to_drop.append(gene)

    df = df.drop(to_drop)
    return df


def gen_chimera(df):
    with open(args.chimeras, 'r') as infile:
        chim_dict = {}
        for line in infile:
            line = line.strip()
            split_line = line.split('\t')
            chim_dict[split_line[0]] = split_line[3:]

    for key in chim_dict.keys():
        df['int_chim'] = 0
        for taxon in chim_dict[key]:
            df['int_chim'] = df['int_chim'] + df[taxon]
            df = df.drop(taxon, axis=1)

        df.loc[df['int_chim'] >= 1, key] = 1
        df.loc[df['int_chim'] < 1, key] = 0
        df = df.drop('int_chim', axis=1)

    return df


def update_dataframe(df):
    """

    :return:
    """
    with open('select_taxa.tsv', 'r') as infile:
        infile.readline()
        to_drop = []
        for line in infile:
            line = line.strip()
            taxon, _, _, _, include = line.split()
            if include == 'no':
                to_drop.append(taxon)

    df = df.drop(to_drop)

    return df


if __name__ == '__main__':
    description = 'Selects taxa to be included in super matrix construction'
    parser, optional, required = help_formatter.initialize_argparse(name='select_taxa.py',
                                                                    desc=description,
                                                                    usage='select_taxa.py '
                                                                          '[OPTIONS]')

    # Optional Arguments
    optional.add_argument('--to_exclude', type=str, metavar='exc_taxa.txt', default=None,
                          help=textwrap.dedent("""\
                          List of taxa to exclude.
                          """))
    optional.add_argument('--to_include', type=str, metavar='inc_taxa.txt', default=None,
                          help=textwrap.dedent("""\
                          List of taxa to include.
                          """))
    optional.add_argument('--chimeras', type=str, metavar='chimeras.tsv', default=None,
                          help=textwrap.dedent("""\
                           A .tsv containing a Unique ID, higher and lower taxonomic designations, 
                           and the Unique IDs of the taxa to collapse, for each chimera one per line
                          """))

    args = help_formatter.get_args(parser, optional, required, inp_dir=False, pre_suf=False, out_dir=False)

    config = configparser.ConfigParser()
    config.read('config.ini')
    dfo = str(Path(config['PATHS']['database_folder']).resolve())
    metadata = f'{dfo}/metadata.tsv'
    orthologs_dir = f'{dfo}/orthologs/'

    matrix = tools.completeness(args=args, input_dir=orthologs_dir, genes=True)
    matrix = matrix.transpose()

    if os.path.isfile('select_orthologs.tsv'):
        matrix = update_df_ortho(matrix)

    if args.chimeras:
        matrix = gen_chimera(matrix)
        matrix.to_csv('text_df.csv')

    gene_count, _ = matrix.shape
    taxa_comp = matrix.sum().divide(other=gene_count)

    make_subset_tsv()
    tools.make_plot(taxa_comp, f'taxa_comp', y_count=gene_count, genes=False)



#!/usr/bin/env python
import configparser
import os
from pathlib import Path

from Bio import SeqIO

from phylofisher import help_formatter


def parse_genes(gene_file):
    """Select gene to exclude: where sgt column is not 'yes' 
    input: csv file with genes
    return: set of genes which should be exluded"""
    to_exlude = set()
    with open(gene_file) as lines:
        next(lines)
        for line in lines:
            gene, _, _, sgt = line.split('\t')
            if sgt.strip().lower() != 'yes':
                to_exlude.add(gene)
    return to_exlude


def parse_orgs(org_file, new_data=False):
    """Parse csv table for organisms selection
    input: csv file with organisms
    return: set of organisms to exclude, set of organisms
    for which we want to select paralogs"""
    to_exlude = set()
    paralogs = set()
    with open(org_file) as lines:
        lines.readline()
        for line in lines:
            if new_data:
                org, _, _, _, _, _, _, _, _, sgt = line.split('\t')
            else:
                org, _, _, _, _, sgt, para = line.split('\t')

            if sgt.strip().lower() != 'yes':
                to_exlude.add(org)

            if not new_data:
                if para.strip().lower() == 'yes':
                    if org not in to_exlude:
                        paralogs.add(org)

        if new_data:
            return to_exlude
        else:
            return to_exlude, paralogs


def fasta_filtr(file, o_to_ex, paralogs=None):
    """Filter fasta sequences with genes. Exludes organisms which should be 
    excluded (o_to_ex)
    input: fasta file with genes, set with orgs to exlude, paralogs=True/None
    result: None"""
    with open(str(Path(args.output, file)), 'w') as res:
        for record in SeqIO.parse(str(Path(args.input, file)), 'fasta'):
            if record.name.split('_')[0] not in o_to_ex:
                res.write(f'>{record.name}\n{record.seq}\n')
        if paralogs:
            # only with paralog selection option
            para_file = str(Path(dfo, f'paralogs/{file.split(".")[0]}_paralogs.fas'))
            if os.path.isfile(para_file):
                for record in SeqIO.parse(para_file, 'fasta'):
                    if record.name.split('.')[0] in paralogs:
                        res.write(f'>{record.name}\n{record.seq}\n')


def main():
    gene_file = str(Path(args.input, 'informant_stats/gene_stats.tsv'))
    new_orgs_file = str(Path(args.input, 'informant_stats/new_taxa_stats.tsv'))
    db_orgs_file = str(Path(args.input, 'informant_stats/db_taxa_stats.tsv'))
    # genes to exclude
    g_to_ex = parse_genes(gene_file)
    # organisms to exlude
    new_o_to_ex = parse_orgs(new_orgs_file, new_data=True)
    db_o_to_ex, paralogs = parse_orgs(db_orgs_file)
    o_to_ex = new_o_to_ex | db_o_to_ex

    # only genes which are not in g_to_ex
    filtered_genes = []
    for file in [file for file in os.listdir(args.input) if file.endswith(f".fas")]:
        if file.split('.')[0] not in g_to_ex:
            filtered_genes.append(file)

    for file in filtered_genes:
        fasta_filtr(os.path.basename(file), o_to_ex, paralogs)


if __name__ == '__main__':
    parser, optional, required = help_formatter.initialize_argparse(name='working_dataset_constructor.py',
                                                                    desc='Script for filtering organisms [and|or] genes',
                                                                    usage='working_dataset_constructor.py [OPTIONS] '
                                                                          '-i <input> ')

    in_help = "Path to output directory of fisher.py containing the output of informant.py"
    args = help_formatter.get_args(parser, optional, required, in_help=in_help, pre_suf=False)

    config = configparser.ConfigParser()
    config.read('config.ini')
    dfo = str(Path(config['PATHS']['database_folder']).resolve())

    if args.input[-1] == '/':
        args.input = args.input[:-1]
    os.mkdir(args.output)
    main()



from setuptools import setup, find_packages

setup(
    name='phylofisher',
    version='1.0.14',
    packages=find_packages(),
    scripts={'phylofisher/fisher.py',
             'phylofisher/config.py',
             'phylofisher/working_dataset_constructor.py',
             'phylofisher/forest.py',
             'phylofisher/matrix_constructor.py',
             'phylofisher/informant.py',
             'phylofisher/apply_to_db.py',
             'phylofisher/install_deps.py',
             'phylofisher/select_taxa.py',
             'phylofisher/select_orthologs.py',
             'phylofisher/sgt_constructor.py',
             'phylofisher/sgt_constructor.smk',
             'phylofisher/prep_final_dataset.py',
             'phylofisher/utilities/purge.py',
             'phylofisher/utilities/build_database.py',
             'phylofisher/utilities/fast_site_remover.py',
             'phylofisher/utilities/mammal_modeler.py',
             'phylofisher/utilities/bipartition_examiner.py',
             'phylofisher/utilities/fast_taxa_remover.py',
             'phylofisher/utilities/aa_comp_calculator.py',
             'phylofisher/utilities/SR4_class_recoder.py',
             'phylofisher/utilities/taxon_collapser.py',
             'phylofisher/utilities/genetic_code_examiner.py',
             'phylofisher/utilities/heterotachy.py',
             'phylofisher/utilities/random_resampler.py',
             'phylofisher/utilities/astral_runner.py',
             'phylofisher/utilities/rtc_binner.py',
             'phylofisher/utilities/backup_restoration.py',
             'phylofisher/utilities/explore_database.py'
             },
    python_requires='==3.7.10',
    install_requires=['biopython==1.78',
                      'pyqt5==5.12.3',
                      'ete3==3.1.2',
                      'pandas==1.2.4',
                      'matplotlib==3.4.1',
                      'scipy==1.6.2',
                      'numpy==1.20.2'
                      ],
    url='https://github.com/TheBrownLab/PhyloFisher',
    license='MIT',
    author='David Zihala',
    author_email='zihaladavid@gmail.com',
    description='PhyloFisher is a software package for the creation, analysis, and visualization of phylogenomic '
                'datasets that consist of protein sequences from eukaryotic organisms.'
)



#!/usr/bin/env python
import configparser
import os
import shutil
import subprocess
import textwrap
from collections import defaultdict
from glob import glob
from pathlib import Path

import matplotlib.pyplot as plt
import pandas as pd
from Bio import AlignIO
from Bio import SeqIO
from Bio.Blast import NCBIXML
from Bio.Data import CodonTable
from matplotlib.backends.backend_pdf import PdfPages

from phylofisher import help_formatter

plt.style.use('ggplot')


def prepare_alignments(threads):
    """
    Prepare multiple sequence alignments from orthologs from the dataset.
    """
    alignmnets_folder = str(Path(dfo, 'alignments'))
    if not os.path.isdir(alignmnets_folder):
        os.mkdir(alignmnets_folder)
    for file in glob(str(Path(dfo, f'orthologs/*.fas'))):
        cmd = f'mafft --auto --reorder --anysymbol --thread {threads} \
            {file} > {file.replace("orthologs", "alignments")}.aln'
        print(cmd)
        subprocess.call(cmd, shell=True)


def parse_query(alignment_file, queries_list):
    """
    Get query sequence (according to input queries) for a given gene. 
    """
    gene = alignment_file.split('/')[-1].split('.')[0]
    ali_dict = {}
    n = 0
    for record in SeqIO.parse(alignment_file, 'fasta'):
        seq = str(record.seq).replace('-', '')
        ali_dict[record.name] = f'>{record.name}@{gene}@{n}\n{seq}\n'
        n += 1
    for query in queries_list:
        if query in ali_dict:
            return ali_dict[query]
    return None


def collect_queries(queries_list):
    """
    Collect query sequences which will be used for blast searches.
    """
    with open('queries.fasta', 'w') as res:
        for file in glob(str(Path(dfo, f'alignments/*.aln'))):
            parsed_query = parse_query(file, queries_list)
            if parsed_query:
                res.write(parsed_query)


def blastDB_tblastn(trans, threads, evalue):
    """
    Make blastdb and perform tblastn with queries from the dataset against given transcriptome.
    """
    subprocess.call(f'makeblastdb -in {trans} -dbtype nucl',
                    shell=True)
    subprocess.call(f'tblastn -query queries.fasta -db {trans} -num_threads {threads} \
                    -evalue {evalue} -outfmt 5 -out blast_out.xml -max_target_seqs 5 ', shell=True)


def read_transcriptome(transcriptome):
    """
    Parse transcriptome as a dictionary.
    """
    result_dict = {}
    for sequence in SeqIO.parse(transcriptome, 'fasta'):
        result_dict[sequence.name] = sequence.seq
    return result_dict


def alignment_parsing(alignment_file, query_n):
    """
    Read multiple sequence aligment and keep information about alignment
    and true positions for all positions for a givne query sequence.
    """
    alignment = AlignIO.read(alignment_file, 'fasta')
    position_dict = defaultdict(list)
    aligment_position = 1
    true_position = 1
    for i in range(alignment.get_alignment_length()):
        if alignment[query_n, i] != '-':
            position_dict[true_position].append(aligment_position)
            position_dict[true_position].append(alignment[:, i])
            true_position += 1
        aligment_position += 1
    return position_dict


def collect_counts(transcriptome, conservation_lvl):
    """
    For every blast record (every gene) check all good quality positions fron alignments
    (avoid first 5 and last 5 amino acids from the alignment; check only
    positions with less than 3 mismatches in their close proximity [-3:+4]) 
    Query position opposite to good qulity position from transcriptome is investigated in
    multiple sequence alignment (prepared only from orthologs from the dataset). In a case,
    when some amino acid is conserved in more then conservation_lvl (70% by default) organims,
    information about conserved amino acid is stored for a corresponding codon in investigated
    transcriptome.
    return: dict of codons and corresponding conserved amino acids dict[UAG] = [Q,Q,Q,E,Q,Q] 
    """

    transcriptome_dict = {}
    for sequence in SeqIO.parse(transcriptome, 'fasta'):
        transcriptome_dict[sequence.name] = sequence.seq

    blastout = open('blast_out.xml')
    blast_records = NCBIXML.parse(blastout)

    AA = 'RHKDESTNQCGPAVILMFYW-'

    result_dict = defaultdict(list)

    for record in blast_records:
        gene_name = record.query.split('@')[1]
        alignment_name = str(Path(dfo, f'alignments/{gene_name}.fas.aln'))
        seq_num = int(record.query.split('@')[-1])
        positional_dict = alignment_parsing(alignment_name, seq_num)
        if len(record.alignments) > 0:
            for hsp in record.alignments[0].hsps:
                hit_name = record.alignments[0].hit_def.split()[0]
                query_start = hsp.query_start
                hit_start = hsp.sbjct_start
                hit_end = hsp.sbjct_end
                frame = hsp.frame[1]
                hit_seq = str(hsp.sbjct)
                query_seq = str(hsp.query)
                middline = str(hsp.match)
                positions = [i + 5 for i in range(len(hit_seq[5:-5]))]
                for position in positions:
                    if "-" not in query_seq[position - 3:position + 4]:
                        if int(middline[position - 3:position + 4].count(' ')) < 3:
                            AA_count = {}
                            real_position = int(query_start + position
                                                - int(query_seq[:position].count('-')))

                            if frame in [1, 2, 3]:
                                real_trans_position = int(
                                    hit_start + (3 * (position - int(hit_seq[:position].count('-')))) - 1)
                                codon = (
                                    transcriptome_dict[hit_name][real_trans_position:real_trans_position + 3]).upper()
                            else:
                                sequence = transcriptome_dict[hit_name]
                                real_trans_position = int(len(sequence) - hit_end + (
                                        3 * (position - int(hit_seq[:position].count('-')))))
                                revcom = sequence.reverse_complement()
                                codon = (revcom[real_trans_position:real_trans_position + 3]).upper()
                            col = str(positional_dict[real_position][1])
                            for amino_acid in AA:
                                AA_count[amino_acid] = col.count(amino_acid)
                            y = pd.Series(AA_count)
                            if y.max() > len(col) * conservation_lvl:
                                result_dict[str(codon)].append(y.idxmax())

    return result_dict


def genecode_plot(res_list_dict, all_codons, transcriptome):
    """
    Prepare bar plot for all codons which seem to differ from the standard  genetic code.
    """
    std_code = CodonTable.unambiguous_dna_by_id[1].forward_table
    std_code['TAG'] = "*"
    std_code['TAA'] = "*"
    std_code['TGA'] = "*"

    with PdfPages(transcriptome + '_genecode.pdf') as pdf:
        AA_ = 'RHKDESTNQCGPAVILMFYW'

        for codon, res_list in res_list_dict.items():
            most_freq = (0, None)
            result = {}
            for i in AA_:
                count = res_list.count(i)
                if count > most_freq[0]:
                    most_freq = (count, i)
                result[i] = res_list.count(i)
                res = pd.Series(result)
            if all_codons:
                if (codon in std_code) and (most_freq[1] != std_code[codon]):
                    print(
                        f"Suspicious codon:{codon.replace('T', 'U')} doens't seem to match The standart genetic code: {std_code[codon]}")
                final_result = res.sort_values(ascending=False)
                final_result.plot.bar()
                plt.title(f"{codon.replace('T', 'U')} (The Standard Code: {std_code[codon]})")
                plt.xticks(rotation='horizontal')
                pdf.savefig(bbox_inches='tight')
                plt.close()
            else:
                if (codon in std_code) and (most_freq[1] != std_code[codon]):
                    print(
                        f"Suspicious codon:{codon.replace('T', 'U')} doens't seem to match The standart genetic code: {std_code[codon]}")
                    final_result = res.sort_values(ascending=False)
                    final_result.plot.bar()
                    plt.title(f"{codon.replace('T', 'U')} (The Standard Code: {std_code[codon]})")
                    plt.xticks(rotation='horizontal')
                    plt.ylabel('Conserved Positions')
                    plt.xlabel('IUPAC Amino Acid Code')
                    pdf.savefig(bbox_inches='tight')
                    plt.close()


def check_alignments():
    """
    Check it aligments folder exist and is not empty.
    """
    alignmnets_folder = str(Path(dfo, 'alignments'))
    if os.path.isdir(alignmnets_folder) == False:
        print("ERROR: For the first run of genetic_code_examiner.py you have to use --prepare_alignments option.")
        return False

    if len(os.listdir(alignmnets_folder)) == 0:
        print("ERROR: Aligments folder is empty. Try to run this script again using --prepare_alignments option.")
        return False
    
    return True

        

def main():
    if args.prepare_alignments:
        prepare_alignments(args.threads)
    if check_alignments(): # execute the block only if alignments are ok
        transcriptome = args.input
        folder = f"{transcriptome.split('.')[0]}_genecode"
        os.mkdir(folder)
        subprocess.call(f'cp {transcriptome} {folder}/', shell=True)
        os.chdir(folder)
        queries_list = args.queries.split(',')
        collect_queries(queries_list)
        print(f'{"=" * 20}\nPerforming blast searches ...\n{"=" * 20}\n')
        blastDB_tblastn(transcriptome, args.threads, args.blast_evalue)
        print(f'{"=" * 20}\nCollecting information from conserved positions ...\n{"=" * 20}\n')
        stats = collect_counts(transcriptome, args.conserved)
        genecode_plot(stats, args.all_codons, transcriptome)
        os.chdir('..')
        if not args.keep_tmp:
            subprocess.call(f'mv {folder}/{transcriptome}_genecode.pdf .', shell=True)
            shutil.rmtree(folder)


if __name__ == '__main__':
    description = 'Script for fast genetic code analysis.'
    usage = 'genetic_code_examiner.py [OPTIONS] -i, --input file.fas -q, --queries Allomacr,Mantplas,...'
    parser, optional, required = help_formatter.initialize_argparse(name='genetic_code_examiner.py',
                                                                    desc=description,
                                                                    usage=usage)

    # Required Arguments
    required.add_argument('-i', '--input', required=True, type=str, metavar='file.fas',
                          help=textwrap.dedent("""\
                          Fasta file with nucleotide sequences.
                          """))
    required.add_argument('-q', '--queries', required=True, type=str, metavar='Allomacr,Mantplas,...',
                          help=textwrap.dedent("""\
                          Comma separated short names of organisms which should be used as queries.
                          """))
    # Optional Arguments
    optional.add_argument('-t', '--threads', type=int, metavar='N', default=1,
                          help=textwrap.dedent("""\
                          Number of threads, where N is an integer.
                          Default: 1
                          """))
    optional.add_argument('--prepare_alignments', action='store_true',
                          help=textwrap.dedent("""\
                          Prepare alignments for genetic code analysis.
                          """))
    optional.add_argument('-c', '--conserved', type=float, metavar='0-1', default=0.7,
                          help=textwrap.dedent("""\
                          Conservation level. 0-1.
                          Default: 0.7 (70%%)
                          """))
    optional.add_argument('-e', '--blast_evalue', type=str, metavar='1e-X', default='1e-30',
                          help=textwrap.dedent("""\
                          Evalue for blast searches. Default: 1e-30.
                          Default: 1e-30
                          """))
    optional.add_argument('-a', '--all_codons', action='store_true', default=False,
                          help=textwrap.dedent("""\
                          Plot conserved positions for all codons.
                          """))
    optional.add_argument('--keep_tmp', action='store_true',
                          help=textwrap.dedent("""\
                          Keep temporary files
                          """))

    args = help_formatter.get_args(parser, optional, required, pre_suf=False, inp_dir=False, out_dir=False)

    config = configparser.ConfigParser()
    config.read('config.ini')
    dfo = str(Path(config['PATHS']['database_folder']).resolve())

    main()



#!/usr/bin/env python
import hashlib
import os
import platform
import shutil
import subprocess
import sys
import tarfile
import textwrap
import time
import urllib.request

from phylofisher import help_formatter


def reporthook(count, block_size, total_size):
    global start_time
    if count == 0:
        start_time = time.time()
        return
    duration = time.time() - start_time
    progress_size = int(count * block_size)
    speed = int(progress_size / (1024 * duration))
    percent = int(count * block_size * 100 / total_size)
    sys.stdout.write("\r%d%%, %d MB, %d KB/s, %d seconds elapsed" %
                    (percent, progress_size / (1024 * 1024), speed, duration))
    sys.stdout.flush()


def bash(cmd):
    subprocess.run(cmd, shell=True)


def is_in_path(cmd):
    """
    Checks to see if command provided is in PATH already. If it is return True, if not returns False
    """
    if shutil.which(cmd) is None:
        in_path = False
    else:
        in_path = True

    return in_path


def download(url):
    fname = url.split('/')[-1]
    urllib.request.urlretrieve(url, f'{fisher_dir}/{fname}', reporthook)

    return fname


def extract(fname):
    tar = tarfile.open(fname, "r:gz")
    tar.extractall()
    tar.close()
    os.remove(fname)


def get_trimal():
    os.chdir(fisher_dir)
    if is_in_path('trimal') is False:
        os.chdir(f'{fisher_dir}/trimAl/source/')
        bash('make')
        os.chdir(fisher_dir)
        # Symlink executables to user bin
        src = f'{fisher_dir}/trimAl/source/trimal'
        des = f'{user_bin}/trimal'
        shutil.copy(src, des)
        src = f'{fisher_dir}/trimAl/source/readal'
        des = f'{user_bin}/readal'
        shutil.copy(src, des)


def get_raxml():
    os.chdir(fisher_dir)
    if is_in_path('raxmlHPC-PTHREADS-AVX2') is False:
        # Download and extract RAxML
        os.chdir(f'{fisher_dir}/standard-RAxML-8.2.12/')
        bash('make -f Makefile.AVX.PTHREADS.gcc')
        # Symlink executables to user bin
        src = f'{fisher_dir}/standard-RAxML-8.2.12/raxmlHPC-PTHREADS-AVX'
        des = f'{user_bin}/raxmlHPC-PTHREADS-AVX2'
        shutil.copy(src, des)


def get_hmmer():
    os.chdir(fisher_dir)
    if is_in_path('hmmsearch') is False:
        # Download and extract hmmer
        os.chdir(f'{fisher_dir}/hmmer-3.3/')
        bash('./configure --prefix=$HOME/.local && make install')


def get_diamond():
    os.chdir(fisher_dir)
    if is_in_path('diamond') is False:
        # Download and extract Diamond
        shutil.copy(f'{fisher_dir}/diamond', f'{user_bin}/diamond')
        os.remove(f'{fisher_dir}/diamond')


def get_fasttree():
    os.chdir(fisher_dir)
    if is_in_path('fasttree') is False:
        # Download and extract Diamond
        bash('gcc -O3 -finline-functions -funroll-loops -Wall -o FastTree FastTree.c -lm')
        shutil.copy(f'{fisher_dir}/FastTree', f'{user_bin}/fasttree')
        os.remove(f'{fisher_dir}/FastTree.c')


def get_blast():
    os.chdir(fisher_dir)
    if is_in_path('blastn') is False:
        if platform.system() == 'Darwin':
            extract(f'{fisher_dir}/ncbi-blast-2.10.1-macosx.tar.gz')
        else:
            extract(f'{fisher_dir}/ncbi-blast-2.10.1-linux.tar.gz')

        src = f'{fisher_dir}/ncbi-blast-2.10.1+/bin/*'
        bash(f'cp {src} {user_bin}')


def get_cd_hit():
    os.chdir(fisher_dir)
    if is_in_path('cd-hit') is False:
        os.chdir(f'{fisher_dir}/cdhit-4.8.1')
        if platform.system() == 'Darwin':
            bash(f'make clean && make CC={args.gxx}')
        else:
            bash('make clean && make')
        exes = ['cd-hit', 'cd-hit-est', 'cd-hit-2d', 'cd-hit-est-2d', 'cd-hit-div', 'cd-hit-454']
        for exe in exes:
            src = f'{fisher_dir}/cdhit-4.8.1/{exe}'
            des = f'{user_bin}/{exe}'
            shutil.copy(src, des)


def get_mafft():
    os.chdir(fisher_dir)
    if is_in_path('mafft') is False:
        os.chdir(f'{fisher_dir}/mafft-7.453-without-extensions/core')
        with open('Makefile', 'r') as infile, open('tmp', 'w') as outfile:
            for line in infile:
                if line.strip() == 'PREFIX = /usr/local':
                    line = f'PREFIX = {fisher_dir}\n'
                elif line.strip() == 'BINDIR = $(PREFIX)/bin':
                    line = f'BINDIR = {user_bin}\n'
                outfile.write(f'{line}')
        print('done')
        shutil.move('tmp', 'Makefile')
        bash('make clean && make && make install')
        os.chdir(fisher_dir)


def get_divvier():
    os.chdir(fisher_dir)
    if is_in_path('divvier') is False:
        os.chdir(f'{fisher_dir}/Divvier-1.01')
        bash('make clean && make')
        src = f'{fisher_dir}/Divvier-1.01/divvier'
        des = f'{user_bin}/divvier'
        shutil.copy(src, des)


def get_prequal():
    os.chdir(fisher_dir)
    if is_in_path('prequal') is False:
        os.chdir(f'{fisher_dir}/prequal-1.02')
        bash('make clean && make')
        src = f'{fisher_dir}/prequal-1.02/prequal'
        des = f'{user_bin}/prequal'
        shutil.copy(src, des)


def get_bmge():
    os.chdir(fisher_dir)
    if is_in_path('BMGE') is False:
        os.chdir(f'{fisher_dir}/BMGE-1.12')
        with open('stub.sh', 'w') as outfile:
            stub = ('#!/bin/sh\n'
                    'MYSELF=`which "$0" 2>/dev/null`\n'
                    '[ $? -gt 0 -a -f "$0" ] && MYSELF="./$0"\n'
                    'java=java\n'
                    'if test -n "$JAVA_HOME"; then\n'
                    '    java="$JAVA_HOME/bin/java"\n'
                    'fi\n'
                    'java_args=-Xmx1g\n'
                    'exec "$java" $java_args -jar $MYSELF "$@"\n'
                    'exit 1\n')
            outfile.write(stub)

        bash('cat stub.sh BMGE.jar > BMGE && chmod +x BMGE ')
        src = f'{fisher_dir}/BMGE-1.12/BMGE'
        des = f'{user_bin}/bmge'
        shutil.copy(src, des)


def get_dist_est():
    os.chdir(fisher_dir)
    if is_in_path('dist_est') is False:
        os.chdir(f'{fisher_dir}/dist_est-1.1')
        bash('make clean && make')
        src = f'{fisher_dir}/dist_est-1.1/dist_est'
        des = f'{user_bin}/dist_est'
        shutil.copy(src, des)


if __name__ == '__main__':
    description = 'Downloads, compiles, and installs trimal, RAxML, hmmer, diamond, FastTree, BLAST+, cd-hit' \
                  'mafft, Divvier, prequal, BMGE, and dist_est'
    parser, optional, required = help_formatter.initialize_argparse(name='install_deps.py',
                                                                    desc=description,
                                                                    usage='install_deps.py [OPTIONS]')

    optional.add_argument('--gxx', type=str, metavar='<path/to/g++>', default=None,
                          help=textwrap.dedent("""\
                            FOR OSX ONLY: Path to g++ compiler.
                            If not provided on OSX, cd-hit installation will be skipped 
                            """))

    args = help_formatter.get_args(parser, optional, required, pre_suf=False, inp_dir=False, out_dir=False)

    # home = os.path.expanduser('~')
    home = os.getcwd()

    user_bin = f'{home}/.local/bin'
    fisher_dir = f'{home}/.local/lib/phylofisher'

    # Check if necessary directy structure is already present.
    # If not creates it
    if os.path.isdir(f'{home}/.local') is False:
        os.mkdir(f'{home}/.local')
    if os.path.isdir(f'{home}/.local/lib') is False:
        os.mkdir(f'{home}/.local/lib')
    if os.path.isdir(user_bin) is False:
        os.mkdir(user_bin)
    if os.path.isdir(fisher_dir) is False:
        os.mkdir(fisher_dir)

    os.chdir(fisher_dir)

    # Get tarball with deps from msstate library
    print('Downloading dependencies:')
    url = 'https://ir.library.msstate.edu/bitstream/handle/11668/19731/Tice_etal.PhyloFisher.archives.tar.gz'
    file_name = 'Tice_etal.PhyloFisher.archives.tar.gz'

    if not os.path.isfile(file_name):
        download(url)

    # Correct original md5 goes here
    original_md5 = '062c84f27388304094d35946f5616a79'

    # Open,close, read file and calculate MD5 on its contents
    with open(file_name, 'rb') as file_to_check:
        # read contents of the file
        data = file_to_check.read()
        # pipe contents of the file through
        md5_returned = hashlib.md5(data).hexdigest()

    # Finally compare original MD5 with freshly calculated
    if original_md5 != md5_returned:
        print('download corrupted')
    else:
        extract(file_name)
        files = os.listdir(f'{fisher_dir}/archives')
        for file in files:
            shutil.move(f'{fisher_dir}/archives/{file}', f'{fisher_dir}/{file}')
            if 'blast' not in file:
                extract(f'{fisher_dir}/{file}')

        shutil.rmtree(f'{fisher_dir}/archives')



    get_trimal()
    get_raxml()
    get_hmmer()
    get_diamond()
    get_fasttree()
    get_blast()
    if platform.system() == 'Darwin' and args.gxx is None:
        print('Your operating system is OSX, and a g++ compiler was not provided.\n'
              'cd-hit will NOT be installed')
    else:
        get_cd_hit()
    get_mafft()
    get_divvier()
    get_prequal()
    get_bmge()
    get_dist_est()



#!/usr/bin/env python
import textwrap

from Bio import SeqIO
from Bio.Seq import Seq

from phylofisher import help_formatter

if __name__ == '__main__':
    description = 'Recodes input matrix based on SR4 amino acid classification.'
    parser, optional, required = help_formatter.initialize_argparse(name='SR4_class_recoder.py',
                                                                    desc=description,
                                                                    usage='SR4_class_recoder.py [OPTIONS] '
                                                                          '-i <matrix>')
    # Required Arguments
    required.add_argument('-i', '--input', required=True, type=str, metavar='matrix',
                          help=textwrap.dedent("""\
                                    Path to input matrix for recoding.
                                    """))
    # Optional Arguments
    optional.add_argument('-if', '--in_format', metavar='<format>', type=str, default='fasta',
                          help=textwrap.dedent("""\
                                    Input file format if not FASTA.
                                    Options: fasta, phylip (names truncated at 10 characters), 
                                    phylip-relaxed (names are not truncated), or nexus.
                                    Default: fasta
                                    """))
    optional.add_argument('-of', '--out_format', metavar='<format>', type=str, default='fasta',
                          help=textwrap.dedent("""\
                                    Desired output format.
                                    Options: fasta, phylip (names truncated at 10 characters), 
                                    phylip-relaxed (names are not truncated), or nexus.
                                    Default: fasta
                                    """))

    args = help_formatter.get_args(parser, optional, required, pre_suf=False, inp_dir=False)

    # SR4 class coding
    key = {'A': ['A', 'G', 'N', 'P', 'S', 'T'],
           'C': ['C', 'H', 'W', 'Y'],
           'G': ['D', 'E', 'K', 'Q', 'R'],
           'T': ['F', 'I', 'L', 'M', 'V']
           }

    out_dict = {'fasta'         : 'fas',
                'phylip'        : 'phy',
                'phylip-relaxed': 'phy',
                'nexus'         : 'nex'}

    # Opens input and output files
    with open(args.input, 'r') as infile, open(f'{args.output}.{out_dict[args.out_format]}', 'w') as outfile:
        all_records = []
        # SeqIO parses input file
        for record in SeqIO.parse(infile, format=args.in_format):
            # Iterates through the key dictionary's keys
            for nuc in key.keys():
                # Iterates through key's values
                for aa in key[nuc]:
                    # Replaces aa with nuc and X's with -'s
                    sequence = str(record.seq).replace(aa, nuc).replace("X", "-")
                    # Creates new SeqIO record
                    record.seq = Seq(sequence)
                    # Appends newly created SeqIO record to a list of SeqIO records to be used by SeqIO.write()
                    all_records.append(record)

        # Writes sequences to the outfile in user specified output format
        SeqIO.write(sequences=all_records, handle=outfile, format=args.out_format)



#!/usr/bin/env python
import configparser
import glob
import os
import shutil
import tarfile
import textwrap
from collections import defaultdict, Counter
from multiprocessing import Pool
from pathlib import Path

import matplotlib.pyplot as plt
from Bio import SeqIO
from ete3 import Tree, TreeStyle, NodeStyle, TextFace
from matplotlib.backends.backend_pdf import PdfPages

from phylofisher import help_formatter

plt.style.use('ggplot')


def configure_colors():
    color_dict = dict()
    with open(color_conf, 'r') as infile:
        infile.readline()
        for line in infile:
            line = line.strip()
            tax, color = line.split('\t')
            color_dict[tax] = color

    return color_dict


def parse_metadata(metadata, input_metadata=None):
    """
    Parse metadata from dataset and input_metadata (if provided)
    input:  metadata csv file, input metadata csv file (optional)
    return: dictionary with combined metadata, dictionary with taxonomy: color
    """
    color_dict = configure_colors()
    metadata_comb = {}
    for line_ in open(metadata):
        if 'Full Name' not in line_:
            sline = line_.split('\t')
            tax = sline[0].strip()
            group = sline[2].strip()
            sub_tax = sline[3]
            full = sline[1].strip()
            if group not in color_dict or color_dict[group].lower() in ['x', 'xx']:
                color_dict[group] = 'white'
            metadata_comb[tax] = {'Higher Taxonomy': group, 'col': color_dict[group], 'full': full,
                                  'Lower Taxonomy': sub_tax}
    if input_metadata:
        for line in open(input_metadata):
            if "FILE_NAME" not in line:
                metadata_input = line.split('\t')
                tax = metadata_input[2].strip().split('_')[0]
                group = metadata_input[3].strip()
                full = metadata_input[6].strip()
                sub_tax = metadata_input[4]
                metadata_comb[tax] = {'Higher Taxonomy': group, 'col': "white", 'full': full, 'Lower Taxonomy': sub_tax}
    return metadata_comb, color_dict


def suspicious_clades(tree):
    """
    Find suspicious clades (more than 70 bs and more than 2 tax groups)
    input: phylogenetic tree
    output: tuple of tree name and list of suspicious clades 
    """
    t = Tree(tree)
    # midpoint rooted tree
    R = t.get_midpoint_outgroup()
    t.set_outgroup(R)

    supported_clades = []
    for node in t.traverse('preorder'):
        if (node.is_root() is False) and (node.is_leaf() is False):
            # report only clades which encompass less than a half of all oranisms
            if node.support >= 70 and (len(node) < (len(t) - len(node))):
                clade = node.get_leaf_names()
                if len(clade) > 1:  # do we need this statement?
                    supported_clades.append(clade)
    suspicious = []
    for clade in supported_clades:
        groups = set()
        for org in clade:
            # get org name
            if '..' in org:
                org = org.split('..')[0]
            else:
                org = org.split('_')[0]
            groups.add(metadata[org]['Higher Taxonomy'])
        if len(groups) > 1:
            suspicious.append(clade)
    return tree, suspicious


def get_best_candidates(tree_file):
    """
    Check best candidates after trimming.
    example: if q1 doesn't survived -> q2 is ranked as best candidate == ortholog
    input: tree file
    output: set of best candidate sequences
    """
    t = Tree(tree_file)
    top_rank = defaultdict(dict)
    for node in t.traverse('preorder'):
        if node.is_leaf():
            if node.name.count('_') == 3:
                org, __, _, rank = node.name.split('_')
                rank = int(rank[1:-1])
                if org not in top_rank:
                    top_rank[org]['rank'] = rank
                    top_rank[org]['candidate'] = node.name
                else:
                    if rank < top_rank[org]['rank']:
                        top_rank[org]['rank'] = rank
                        top_rank[org]['candidate'] = node.name
    top_seqs = set()
    for org_ in top_rank.values():
        top_seqs.add(org_['candidate'])
    return top_seqs


def parse_contaminants(file):
    """
    Parse contamination file.
    example:
    ==============================
    Pirisoci	Alveolata	Higher Taxonomy
    DiplDSTH	GoniavonGEN	Unique ID
    Fucucera	Ochrophyta	Lower Taxonomy
    ==============================
    input: tab separated file with contaminants
    result: dictionary with organism as key and tuple with taxonomy
            and rank 
    """
    cont_dict = {}
    for line in open(file):
        org, tax, rank = line.split('\t')
        cont_dict[org] = (tax, rank.strip())
    return cont_dict


def expected_neighborhood(parent, cont_key_rank):
    """
    Recursively check if ornanisms around target has some taxonomical group 
    (not added in this run of fisher) and evaluete if target has expected 
    neighbourhood.
    input: parent node, (key, rank) => example: (Alveolata, group) or (Gonianon, org)
    output: True/False
    """
    keywords = set()
    key = cont_key_rank[0]
    rank = cont_key_rank[1]
    # check that rank is valid (group, Lower Taxonomy, org)
    assert rank in ['Higher Taxonomy', 'Lower Taxonomy',
                    'Unique ID'], f'{rank} has to be Higher Taxonomy,Lower Taxonomy or org'
    for org in parent.get_leaf_names():
        if (org.count('_') != 4):
            if '..' in org:
                # in case of paralogs
                org = org.split('..')[0]
            else:
                org = org.split('_')[0]
            if rank == 'Higher Taxonomy':
                keywords.add(metadata[org]['Higher Taxonomy'])
            elif rank == 'Lower Taxonomy':
                keywords.add(metadata[org]['Lower Taxonomy'])
            elif rank == 'Unique ID':
                keywords.add(org)
    if keywords:
        if len(keywords) == 1:
            # only in a case that expected neighbourhood is exactly what we
            # are expecting to be. Only one Higher Taxonomy, Lower Taxonomy, tax
            if key in list(keywords)[0]:
                return True
            else:
                return False
        else:
            return False
    return expected_neighborhood(parent.up, cont_key_rank)


def collect_contaminants(tree_file, cont_dict):
    """
    Collect name of all sequences where position on tree corresponds to expected place 
    for a contamination.
    input: tree file, contamination dict from parse_contaminants fucntion
    result: set of proven contaminants, set of proven contamination (same names as in csv result tables)
    """
    t = Tree(tree_file)
    R = t.get_midpoint_outgroup()
    t.set_outgroup(R)
    cont_table_names = set()
    contaminants = set()
    n = 0
    for node in t.traverse('preorder'):
        if node.is_leaf() is True:
            if node.name.count('_') == 4:
                name = node.name
                org = name.split('_')[0]
                quality = f'{node.name.split("_")[-3]}_{node.name.split("_")[-2]}_{node.name.split("_")[-1]}'
                table_name = f'{metadata[org]["full"]}_{quality}@{org}'
                if org in cont_dict:
                    exp_hood = expected_neighborhood(node.up, cont_dict[org])
                    if exp_hood is True:
                        contaminants.add(name)
                        cont_table_names.add(table_name)
        n += 1
    return contaminants, cont_table_names


def get_build_len(name_):
    if os.path.isfile(f'{args.input}/{name_}.trimmed') is True:
        with open(f'{args.input}/{name_}.final', 'r') as build, open(f'{args.input}/{name_}.trimmed', 'r') as trimmed:
            trimmed_len = 0
            len_dict = {}
            for record in SeqIO.parse(trimmed, 'fasta'):
                if len(record.seq) > trimmed_len:
                    trimmed_len = len(record.seq)
                len_dict[record.description] = len(str(record.seq).replace('-', '').replace('X', ''))

            build_len = 0
            for record in SeqIO.parse(build, 'fasta'):
                if len(record.seq) > build_len:
                    build_len = len(record.seq)
        return build_len, len_dict, trimmed_len

    else:
        with open(f'{args.input}/{name_}.final', 'r') as infile:
            build_len = 0
            len_dict = {}
            for record in SeqIO.parse(infile, 'fasta'):
                if len(record.seq) > build_len:
                    build_len = len(record.seq)
                len_dict[record.description] = len(str(record.seq).replace('-', '').replace('X', ''))
        return build_len, len_dict


def tree_to_tsvg(tree_file, contaminants=None, backpropagation=None):
    if contaminants is None:
        contaminants = set()
    tree_base = str(os.path.basename(tree_file))

    # what if they will use somethig different than Raxml? We should make some if statement here maybe.
    name_ = tree_base.split('.')[1]

    if os.path.isfile(f'{args.input}/{name_}.trimmed') is True:
        build_len, len_dict, trimmed_len = get_build_len(name_)
        len_info = f'Final Align Len: {build_len}, Trimmed Align Len: {trimmed_len}'
        len_dict = {k: round(v / trimmed_len, 2) for k, v in len_dict.items()}
    else:
        build_len, len_dict = get_build_len(name_)
        len_info = f'Final Align Len: {build_len}'
        len_dict = {k: round(v / build_len, 2) for k, v in len_dict.items()}

    if not backpropagation:
        table = open(f"{output_folder}/{name_.split('_')[0]}.tsv", 'w')
    else:
        table = open(f"{output_folder}/{name_.split('_')[0]}.tsv", 'r')

    top_ranked = get_best_candidates(tree_file)
    t = Tree(tree_file)
    ts = TreeStyle()
    R = t.get_midpoint_outgroup()
    t.set_outgroup(R)
    sus_clades = 0

    for node in t.traverse('preorder'):
        node_style = NodeStyle()
        node_style['vt_line_width'] = 3
        node_style['hz_line_width'] = 3
        node_style['vt_line_type'] = 0
        node_style['hz_line_type'] = 0

        if node.is_root() is False:
            if node.is_leaf() is False:
                # All internal nodes
                supp, sus_clades = format_nodes(node, node_style, sus_clades, t)
                node.add_face(supp, column=0, position="branch-bottom")
                node.set_style(node_style)
            else:
                # All leaves
                format_leaves(backpropagation, contaminants, node, node_style, table, top_ranked, len_dict)
                node.set_style(node_style)

    title_face = TextFace(f'<{name_}  {len_info}, {sus_clades} suspicious clades>', bold=True)
    ts.title.add_face(title_face, column=1)
    t.render(f'{output_folder}/{name_}_tree.svg', tree_style=ts, )
    if not backpropagation:  # what what what?
        table.close()


def format_leaves(backpropagation, contaminants, node, node_style, table, top_ranked, len_dict):
    """This function formats the leaves for the svg file and writes to tsv file"""

    # This parts is about leaves
    empty_face = TextFace("\t" * 20)  # just because of Sefs script
    node.add_face(empty_face, column=2, position="aligned")
    node.add_face(empty_face, column=3, position="aligned")
    org = node.name
    if org.count('_') == 3:  # if org in additions
        org_in_add(backpropagation, contaminants, node, org, table, top_ranked, len_dict)

    elif '..' in org:
        para_from_meta(backpropagation, node, org, table, len_dict)

    else:
        ortho_from_meta(backpropagation, node, node_style, org, table, len_dict)


def org_in_add(backpropagation, contaminants, node, org, table, top_ranked, len_dict):
    """for organisms that are in additions"""
    quality = f'{org.split("_")[-2]}_{org.split("_")[-1]}_{len_dict[org]}'
    org = node.name
    unique_id = org.split('_')[0]
    group = metadata[unique_id]['Higher Taxonomy']
    if group in tax_col:
        tax_name = TextFace(f'[{group} {metadata[unique_id]["Lower Taxonomy"]}]', fgcolor=tax_col[group], bold=True)
    else:
        tax_name = TextFace(f'[{group} {metadata[unique_id]["Lower Taxonomy"]}]', bold=True)
    node.add_face(tax_name, column=1, position="aligned")
    if node.name in contaminants:
        # Seqs to delete
        if not backpropagation:
            table.write(f'{metadata[unique_id]["full"]}_{quality}@{unique_id}\t{group}\td\n')
        delf = TextFace(f'{metadata[unique_id]["full"]}_{quality}@{unique_id}', fgcolor='red')
        node.name = ''
        node.add_face(delf, column=0)
    elif node.name in top_ranked:
        # top ranked guys
        tname = TextFace(f'{metadata[unique_id]["full"]}_{quality}@{unique_id}', bold=True)
        if not backpropagation:
            table.write(f'{metadata[unique_id]["full"]}_{quality}@{unique_id}\t{group}\to\n')
        node.name = ''
        node.add_face(tname, column=0, position='branch-right')
    else:
        if not backpropagation:
            table.write(f'{metadata[unique_id]["full"]}_{quality}@{unique_id}\t{group}\tp\n')
        node.name = f'{metadata[unique_id]["full"]}_{quality}@{unique_id}'


def para_from_meta(backpropagation, node, org, table, len_dict):
    """for paralogs from metadata"""
    unique_id = org.split('..')[0]
    group = f"{metadata[unique_id]['Higher Taxonomy']}"
    paraf = TextFace(f'{metadata[unique_id]["full"]}_{len_dict[org]}@{org}', fgcolor='blue')
    node.name = ''
    node.add_face(paraf, column=0)
    if not backpropagation:
        table.write(f'{metadata[unique_id]["full"]}_{len_dict[org]}@{org}\t{group}\tp\n')
    gface = TextFace(f'[{group} {metadata[unique_id]["Lower Taxonomy"]}]')
    node.add_face(gface, column=1, position="aligned")


def ortho_from_meta(backpropagation, node, node_style, org, table, len_dict):
    """for orthologs from dataset"""
    group = f"{metadata[org]['Higher Taxonomy']}"
    gface = TextFace(f'[{group} {metadata[org]["Lower Taxonomy"]}]')  # TODO do not touch me pleeeease
    color = metadata[org]['col']
    node_style["bgcolor"] = color
    if not backpropagation:
        table.write(f'{metadata[org]["full"]}_{len_dict[org]}@{org}\t{group}\to\n')
    node.name = f'{metadata[org]["full"]}_{len_dict[org]}@{org}'
    node.add_face(gface, column=1, position="aligned")


def format_nodes(node, node_style, sus_clades, t):
    """This function visually formats the nodes in the svg file based on the nodes' support values and whether or not
    the clade is suspicous"""
    supp = TextFace(f'{int(node.support)}', fsize=8)
    if node.support >= 70:
        supp.bold = True
        taxons = set()
        orgs = node.get_leaf_names()
        if len(orgs) > 1:
            for org in orgs:
                if '..' in org:  # for paralogs
                    org = org.split('..')[0]
                else:
                    org = org.split('_')[0]  # for potential orthologs
                taxons.add(metadata[org]['Higher Taxonomy'])
        if len(taxons) > 1 and (len(node) < (len(t) / 2)):
            node_style['shape'] = 'sphere'
            node_style['size'] = 12
            node_style['fgcolor'] = 'red'
            node_style['bgcolor'] = 'Silver'
            sus_clades += 1
    else:
        supp.fsize = 7
    return supp, sus_clades


def parallel_susp_clades(trees):
    """Parallelizes the function suspicious_clades()"""
    with Pool(processes=threads) as pool:
        suspicious = list(pool.map(suspicious_clades, trees))
        return suspicious


def nonredundant(result_clades):
    nonredundant = []
    for clades in result_clades:
        if clades:
            sorted_clades = sorted(clades, key=lambda clade: len(clade))
            sorted_sets = [set(clade) for clade in sorted_clades]
            for i, set_ in enumerate(sorted_sets):
                redundant = False
                if i < len(sorted_sets):
                    for other in sorted_sets[i + 1:]:
                        if set_.issubset(other):
                            redundant = True
                if redundant is False:
                    nonredundant.append(set_)
    return nonredundant


def collect_major_taxa(nonredundant_clades_):
    """
    Collect information about major taxonomic group for all nonreduntant clades
    and connect this information with all organisms in that clade. This function
    prepares data for plot_major_taxa function.
    input: set of nonredundant clades
    return: dictionary with orgs as keys and list of taxonomic groups (major tax groups
    from clades with a given organism) as values
    """
    orgs_ = defaultdict(list)
    for clade in nonredundant_clades_:
        # collect all taxonomic groups for a given clade
        taxons = []
        for org in clade:
            if '..' in org:
                # for paralogs
                org = org.split('..')[0]
            else:
                org = org.split('_')[0]
            taxons.append(metadata[org]["Higher Taxonomy"])
        group_perc = {}
        for tax in set(taxons):
            group_perc[tax] = (taxons.count(tax) / len(taxons)) * 100
        max_perc = max(group_perc, key=group_perc.get)
        for seq in clade:
            # parse names again
            if '..' in seq:
                # paralogs
                org = seq.split('..')[0]
            else:
                org = seq.split('_')[0]
            if group_perc[metadata[org]["Higher Taxonomy"]] == 50:
                orgs_[org].append('unspecified')
            else:
                orgs_[org].append(max_perc)
    return orgs_


def plot_major_taxa(orgs_taxa):
    """
    Plot major taxonomic groups from clades (with a given org) for all organisms.
    input: result from collect_major_taxa function (dictionary with list as keys)
    return: None
    """
    max_ = 0
    for org, groups in sorted(orgs_taxa.items()):
        # this 'for loop' is just looking for maximum value of clades
        # with same taxonomic group for one organism. This number (max_)
        # is then used as maximum in plt.ylim to put all orgs on the same
        # scale
        counted_groups = dict(Counter(groups))
        group_total = 0
        for group in counted_groups.values():
            group_total += group
        if group_total > max_:
            max_ = group_total
    with PdfPages('orgs_taxa.pdf') as pdf:
        for org, groups in sorted(orgs_taxa.items()):
            counted_groups = dict(Counter(groups))
            colors = []
            for group in counted_groups.keys():
                if group == 'unspecified':
                    colors.append('black')
                else:
                    colors.append(tax_col.get(group, "white"))
            plt.bar(counted_groups.keys(), counted_groups.values(), color=colors)
            plt.tight_layout()
            plt.ylim(0, max_)
            plt.title(f'{org}({metadata[org]["Higher Taxonomy"]})')
            plt.xticks(rotation='vertical')
            plt.yticks(fontsize=4)
            pdf.savefig(bbox_inches='tight')
            plt.close()


def backpropagate_contamination(tree_file, cont_names):
    """
    Changes status in all csv result tables for all proven contaminants
    to 'd' as delete.
    input: tree file, set of proven contaminants (same name format as in csv tables)
    result: None
    """
    tree_base = str(os.path.basename(tree_file))
    output_base = f"{tree_base.split('.')[1].split('_')[0]}"
    tree_base = tree_base.split("_")[0]

    # change original tables before parsing
    original_table = open(f"{output_folder}/{output_base}.tsv", 'r').readlines()
    with open(f"{output_folder}/{output_base}.tsv", 'w') as res_:
        for line in original_table:
            sline = line.split('\t')
            name = sline[0]
            tax = sline[1]
            status = sline[2].strip()
            if name in cont_names:
                status = 'd'
            res_.write(f'{name}\t{tax}\t{status}\n')

    # changed already parsed table (*_parsed.tsv) if it already exists
    parsed_table_file = f"{output_folder}/{output_base}_parsed.tsv"
    if os.path.isfile(parsed_table_file):
        parsed_table = open(parsed_table_file, 'r').readlines()
        with open(f"{output_folder}/{output_base}.tsv", 'w') as res_:
            for line in parsed_table:
                sline = line.split('\t')
                name = sline[0]
                tax = sline[1]
                status = sline[2].strip()
                if name in cont_names:
                    status = 'd'
                res_.write(f'{name}\t{tax}\t{status}\n')


if __name__ == '__main__':
    description = 'Render .svg and .tsv files of single gene trees for visualization with ParaSorter.'
    parser, optional, required = help_formatter.initialize_argparse(name='forest.py',
                                                                    desc=description,
                                                                    usage='forest.py [OPTIONS] -i <in_dir>')
    # Add Arguments Specific to this script
    # Optional Arguments
    optional.add_argument('-a', '--contaminants', metavar='<contams.tsv>',
                          help=textwrap.dedent("""\
                          Path to file containing previously known contaminants to be removed."""))
    optional.add_argument('-b', '--backpropagate', action='store_true',
                          help=textwrap.dedent("""\
                          Remove contaminants through backpropagation."""))
    optional.add_argument('-t', '--threads', metavar='<N>', type=int,
                          help=textwrap.dedent("""\
                          Number of threads to be used, where N is an integer. 
                          Default: N=1"""))
    optional.add_argument('--local_run', action='store_true',
                          help=textwrap.dedent("""\
                          To be used when sgt_constructor_out.tar.gz has been downloaded from a server for
                          single gene tree visualizations to be rendered locally.
                          """))

    in_help = 'Path to sgt_constructor_out_<M.D.Y>/trees if run remotely or sgt_constructor_out_<M.D.Y>.tar.gz ' \
              'if run locally'
    args = help_formatter.get_args(parser, optional, required, pre_suf=False, in_help=in_help)

    output_folder = args.output

    if args.local_run:
        with tarfile.open(args.input, "r:gz") as tar:
            tar.extractall()
            tar.close()

        args.input = args.input.split('.tar.gz')[0]
        trees_folder = f'{args.input}/trees'
        args.metadata = f'{args.input}/metadata.tsv'
        args.input_metadata = f'{args.input}/input_metadata.tsv'
        color_conf = f'{args.input}/tree_colors.tsv'

        args.input = f'{args.input}/trees'

    else:
        trees_folder = args.input
        config = configparser.ConfigParser()
        config.read('config.ini')
        dfo = str(Path(config['PATHS']['database_folder']).resolve())
        args.metadata = str(os.path.join(dfo, 'metadata.tsv'))
        color_conf = str(Path(config['PATHS']['color_conf']).resolve())
        args.input_metadata = str(os.path.abspath(config['PATHS']['input_file']))

    if not args.backpropagate:
        os.mkdir(output_folder)

    trees = glob.glob(f"{trees_folder}/*.tre")

    number_of_genes = len(trees)
    metadata, tax_col = parse_metadata(args.metadata, args.input_metadata)
    threads = args.threads

    if not args.backpropagate:
        suspicious = parallel_susp_clades(trees)
        suspicious = sorted(suspicious)

        make_txt = False
        for x, y in suspicious:
            if y:
                make_txt = True

        if make_txt:
            with open('suspicious.txt', 'w') as res:
                for file, clades in suspicious:
                    if clades:
                        res.write(f'{file}\n========================\n')
                        for clade_ in clades:
                            res.write(f'{clade_}\n')
                        res.write(f'\n\n\n')
            result_clades = [clades[1] for clades in suspicious]
            nonredundant_clades = nonredundant(result_clades)
            orgs_taxa = collect_major_taxa(nonredundant_clades)

        # Commented out due to issues with the function
        # plot_major_taxa(orgs_taxa)

    if args.contaminants:
        cont_dict = parse_contaminants(args.contaminants)
        for tree in trees:
            contaminants, contaminated_table = collect_contaminants(tree, cont_dict)
            if args.backpropagate:
                backpropagate_contamination(tree, contaminated_table)
                tree_to_tsvg(tree, contaminants, backpropagation=True)
            else:
                tree_to_tsvg(tree, contaminants)
    else:
        for tree in trees:
            tree_to_tsvg(tree)

    if args.local_run:
        shutil.rmtree('/'.join(args.input.split('/')[:-1]))



#!/usr/bin/env python
import configparser
import glob
import os
import random
import shutil
import string
import tarfile
import textwrap
from collections import defaultdict
from datetime import date
from pathlib import Path

import pandas as pd
from Bio import SeqIO

from phylofisher import help_formatter, tools
from phylofisher.utilities import build_database


def dataset_orgs():
    """"Collect all short names from dataset metadata table."""
    orgs = set()
    with open(metadata) as md:
        # skip header
        next(md)
        for line_ in md:
            sline = line_.split('\t')
            abbrev = sline[0].strip()
            orgs.add(abbrev)
    return orgs


def taxa_to_exclude():
    to_skip = []
    with open(args.to_exclude, 'r') as infile:
        for line in infile:
            unique_id = line.strip().split(',')[0]
            to_skip.append(unique_id)

    return to_skip


def parse_input(input_metadata):
    """"Parse input metadata.
    input: input metadata csv
    return: dictionary with info about input metadata
    """
    orgs_to_exc = set()
    if args.to_exclude:
        orgs_to_exc = taxa_to_exclude()
    input_info = defaultdict(dict)
    with open(input_metadata, 'r') as infile:
        for line in infile:
            line = line.strip()
            sline = line.split('\t')
            abbrev = sline[2]

            if 'Location' in line:
                pass
            elif abbrev in orgs_to_exc:
                pass
            else:
                group = sline[3].strip()
                full_name = sline[6].strip()
                subtax = sline[4]
                data_type = sline[7]
                notes = sline[8].strip()
                input_info[abbrev]['tax'] = group
                input_info[abbrev]['full_name'] = full_name
                input_info[abbrev]['subtax'] = subtax
                input_info[abbrev]['data_type'] = data_type
                input_info[abbrev]['notes'] = notes

    return input_info


def collect_seqs(gene):
    """Collect all sequences for a given gene from fasta folder (fisher result)
    and store them in a dictionary.
    input: gene name
    return: dictionary of all seqs for a given gene
    """
    seq_dict = {}
    for record in SeqIO.parse(f'{fisher_dir}/{gene}.fas', 'fasta'):
        if record.name.count('_') == 3:
            abbrev, _, _, quality = record.name.split('_')
            qname = f'{abbrev}_{quality}'
            seq_dict[qname] = record
        else:
            seq_dict[record.name] = record
    return seq_dict


def id_generator(size=5, chars=string.digits):
    """"Generate random number with 5 digits."""
    return ''.join(random.choice(chars) for _ in range(size))


def paralog_name(abbrev, keys):
    """"Prepare paralog name (short name + 5 random digits).
    Recursive function.
    example: Homosap..12345
    input: short name of an organism, names of already existing paralogs
    for a given organism
    return: unique paralog name"""
    id_ = id_generator()
    pname = f'{abbrev}..p{id_}'
    if pname not in keys:
        return pname
    else:
        paralog_name(abbrev, keys)


def parse_table(table):
    """Collect information what has to be changed in
        the PhyloFisher dataset (for a given gene)
        according to information from
        parsed single gene tree.

     input: tsv table of one gene
     return: dictionaries of new orthologs and paralogs"""

    # gene name
    gene = table.split('/')[-1].split('_parsed')[0]
    # path to orthologs in the dataset folder
    orthologs_path = str(Path(dfo, f'orthologs/{gene}.fas'))
    # path to paralogs in the dataset folder
    paralogs_path = str(Path(dfo, f'paralogs/{gene}_paralogs.fas'))

    # parse orthologs for a given gene into a dictionary
    orthologs = SeqIO.to_dict(SeqIO.parse(orthologs_path, "fasta"))

    if os.path.isfile(paralogs_path):
        # parse paralogs a given gene into a dictionary (if they exist)
        paralogs = SeqIO.to_dict(SeqIO.parse(paralogs_path, "fasta"))
    else:
        # if paralogs doesnt exist, make an empty directory
        paralogs = {}

    # collect all candidate sequences for a given gen from the fisher.py result
    seq_dict = collect_seqs(gene)

    for line in open(table):
        # for orthologs from the dataset
        tree_name, tax, status = line.split('\t')
        status = status.strip()  # o,p,d (ortholog, paralog, delete)
        abbrev = tree_name.split('@')[-1]
        if tree_name.count('_') != 3 and '..' not in abbrev:
            record = seq_dict[abbrev]
            if status == 'd':
                # delete sequence from orthologs
                del orthologs[abbrev]
            elif status == 'p':
                # chance status from paralog to ortholog
                # prepare paralog name
                pname = paralog_name(abbrev, paralogs.keys())
                paralogs[pname] = record
                # delete sequence from orthologs
                del orthologs[abbrev]

    for line in open(table):
        # for new sequences
        tree_name, tax, status = line.split('\t')
        status = status.strip()
        abbrev = tree_name.split('@')[-1]
        if tree_name.count('_') == 3:
            quality = tree_name.split('_')[2]
            qname = f'{abbrev}_{quality}'
            record = seq_dict[qname]
            if status == 'o':
                # add to orthologs
                orthologs[abbrev] = record
            elif status == 'p':
                # add to paralogs
                pname = paralog_name(abbrev, paralogs.keys())
                paralogs[pname] = record

    for line in open(table):
        # for paralogs from the dataset
        tree_name, tax, status = line.split('\t')
        status = status.strip()
        name = tree_name.split('@')[-1]
        abbrev = name.split('.')[0]
        if '..' in name:
            if status == 'o':
                record = paralogs[name]
                # for cases when ortholog has not survived trimming
                if abbrev in orthologs:
                    # prepare paralog name
                    pname = paralog_name(abbrev, paralogs.keys())
                    # chance status from paralog to ortholog
                    paralogs[pname] = orthologs[abbrev]
                    # record in orthologs will be replaced
                    # with a new one in the next step
                # add sequence to orthologs 
                orthologs[abbrev] = record
                # delete sequence from paralogs
                del paralogs[name]

    return orthologs, paralogs


def add_to_meta(abbrev):
    """"Transfer input metadata for a given organism
    to dataset metadata.
    input:  short name of organism
    return: None
    """
    new_row = {'Unique ID': abbrev,
               'Long Name': input_info[abbrev]['full_name'],
               'Higher Taxonomy': input_info[abbrev]['tax'].replace('*', ''),
               'Lower Taxonomy': input_info[abbrev]['subtax'].replace('*', ''),
               'Data Type': input_info[abbrev]['data_type'],
               'Source': input_info[abbrev]['notes']
               }

    df = pd.read_csv(metadata, delimiter='\t')
    df.index = df['Unique ID']

    df = df.append(new_row, ignore_index=True)
    df = df.sort_values(by=['Higher Taxonomy', 'Lower Taxonomy', 'Unique ID'])
    df.to_csv(str(Path(dfo, 'metadata.tsv')), sep='\t', index=False)


def new_database(table):
    """Make changes in the PhyloFisher dataset according to information from
     parsed single gene tree."""

    gene = table.split('/')[-1].split('_parsed')[0]
    # Orthologs for a given gene
    orthologs_path = str(Path(dfo, f'orthologs/{gene}.fas'))
    # Paralogs for a given gene
    paralogs_path = str(Path(dfo, f'paralogs/{gene}_paralogs.fas'))

    orthologs, paralogs = parse_table(table)

    with open(orthologs_path, 'w') as res:
        # make changes in orthologs
        for name, record in orthologs.items():
            if name not in meta_orgs and name not in to_exclude:
                meta_orgs.add(name)
                add_to_meta(name)
            if name.split('..')[0] not in to_exclude:
                res.write(f'>{name}\n{record.seq}\n')

    with open(paralogs_path, 'w') as res:
        # make changes in paralogs
        for name, record in paralogs.items():
            if name.split('..')[0] not in meta_orgs and name.split('..')[0] not in to_exclude:
                meta_orgs.add(name.split('.')[0])
                add_to_meta(name.split('.')[0])
            if name.split('..')[0] not in to_exclude:
                res.write(f'>{name}\n{record.seq}\n')


def rebuild_db():
    """
    
    :return: 
    """
    try:
        shutil.rmtree(f'{dfo}/profiles')
    except FileNotFoundError:
        pass

    try:
        shutil.rmtree(f'{dfo}/datasetdb')
    except FileNotFoundError:
        pass
    
    cwd = os.getcwd()
    os.chdir(dfo)
    args.rename = None
    build_database.main(args, args.threads, True, 0.1)
    os.chdir(cwd)


def cp_proteomes():
    with open(input_metadata, 'r') as infile:
        infile.readline()
        file_dict = {}
        for line in infile:
            s_line = line.strip().split('\t')
            file_dict[s_line[2]] = os.path.abspath(os.path.join(s_line[0], s_line[1]))

    os.mkdir('tmp')
    os.chdir('tmp')
    for key in file_dict.keys():
        with tarfile.open(f'{key}.faa.tar.gz', "x:gz") as tar:
            tar.add(file_dict[key], arcname=f'{key}.faa')
        shutil.copy(f'{key}.faa.tar.gz', f'{dfo}/proteomes/{key}.faa.tar.gz')

    os.chdir('..')
    shutil.rmtree('tmp')


def main():
    """
    Main function. Run new_database on all parsed trees (tsv files)
    :return:
    """
    for table in glob.glob(f'{args.input}/*_parsed.tsv'):
        new_database(table)

    rebuild_db()
    cp_proteomes()


if __name__ == '__main__':
    desc = ('Apply parsing decisions and add new data to the database. \n\n'
            'NOTE: If apply_to_db.py fails for any reason, see backup_resoration.py \n'
            'and restore you database from the proper backup')
    parser, optional, required = help_formatter.initialize_argparse(name='apply_to_db.py',
                                                                    desc=desc,
                                                                    usage="apply_to_db.py -i <in_dir>")

    required.add_argument('-fi', '--fisher_dir', type=str, metavar='<fisher_dir>', required=True,
                          help=textwrap.dedent("""\
                          Path to fisher output directory to use for dataset addition.
                          """))

    optional.add_argument('--to_exclude', type=str, metavar='to_exclude.txt',
                          help=textwrap.dedent("""\
                          Path to .txt file containing Unique IDs of taxa to exclude from dataset 
                          addition with one taxon per line.
                          Example:
                            Unique ID (taxon 1)
                            Unique ID (taxon 2)
                            """))

    optional.add_argument('-t', '--threads', type=int, metavar='N', default=1,
                          help=textwrap.dedent("""\
                          Number of threads, where N is an integer.
                          Default: 1
                          """))

    in_help = 'Path to forest output directory to use for database addition.'
    args = help_formatter.get_args(parser, optional, required, out_dir=False, pre_suf=False, in_help=in_help)

    fisher_dir = args.fisher_dir

    config = configparser.ConfigParser()
    config.read('config.ini')
    dfo = str(Path(config['PATHS']['database_folder']).resolve())

    tools.backup(dfo)

    to_exclude = taxa_to_exclude()
    input_metadata = os.path.abspath(config['PATHS']['input_file'])
    metadata = str(Path(dfo, 'metadata.tsv'))
    meta_orgs = dataset_orgs()
    input_info = parse_input(input_metadata)
    main()



#!/usr/bin/env python
import os
import random
import string
import subprocess
import sys
import textwrap

import pandas as pd
from Bio import SeqIO
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord

from phylofisher import help_formatter


def id_generator(size=10, chars=string.digits):
    return ''.join(random.choice(chars) for _ in range(size))


def unique_name(keys):
    id_ = id_generator()
    if id_ not in keys:
        return id_
    else:
        unique_name(keys)


def fake_phylip(matrix):
    length = None
    pseudonames = {}
    pseudonames_rev = {}
    records = []

    for record in SeqIO.parse(matrix, args.in_format):
        uname = unique_name(pseudonames)
        pseudonames[record.name] = uname
        pseudonames_rev[uname] = record.name
        seq = str(record.seq)
        records.append(SeqRecord(Seq(seq),
                                 id=uname,
                                 name='',
                                 description=''))

    SeqIO.write(records, 'TEMP.phy', 'phylip')

    return pseudonames, pseudonames_rev


def fake_tree(treefile, pseudonames):
    with open('TEMP.tre', 'w') as res:
        original = open(treefile).readline()
        for key, value in pseudonames.items():
            original = original.replace(key, value)
        res.write(original)


def control_file():
    ctl = """treefile = TEMP.tre * treefile
seqfile = TEMP.phy * sequence data

nchar = 20             * amino acid data
model =  3             * empirical + F
aaRatefile = lg.dat * JTT substitution model

nrate = 101            * number of rates
ub = 10.0              * upper bound for rates"""
    with open('dist_est.ctl', 'w') as res:
        res.write(ctl)


def run_dist():
    cmd = 'dist_est dist_est.ctl'
    subprocess.run(cmd, shell=True)


def parse_rates():
    positions = []
    site = 0
    for line in open('rate_est.dat'):
        _, rate, _, _ = line.split()
        positions.append((site, float(rate)))
        site += 1
    sorted_positions = sorted(positions, key=lambda x: x[1], reverse=True)
    result = [i[0] for i in sorted_positions]
    return result


def main():
    pseudo_, pseudo_rev_ = fake_phylip(args.matrix)
    fake_tree(args.tree, pseudo_)
    control_file()
    # Checks to see if dist_est has been run. If not do so
    if not os.path.isfile('./rate_est.dat'):
        run_dist()

    matrix_dict = {}
    for record in SeqIO.parse(args.matrix, args.in_format):
        matrix_dict[record.name] = pd.Series(list(record.seq))

    sorted_rates = parse_rates()
    iter = 0
    os.mkdir(f'{args.output}/steps_{args.step_size}')
    os.chdir(f'{args.output}/steps_{args.step_size}')

    out_dict = {'fasta'         : 'fas',
                'phylip'        : 'phy',
                'phylip-relaxed': 'phy',
                'nexus'         : 'nex'}

    for step in range(args.step_size, len(sorted_rates), args.step_size):
        with open(f'step{iter}.{out_dict[args.out_format.lower()]}', 'w') as res:
            records = []
            for name, seq in matrix_dict.items():
                seq = "".join(seq[sorted_rates[step:]].values)
                records.append(SeqRecord(Seq(seq),
                                         id=name,
                                         name='',
                                         description=''))

            # Writes to output matrix in user specified output
            if args.out_format.lower() in out_dict:
                SeqIO.write(records, res, args.out_format.lower())
            else:
                sys.exit('Invalid Output Format')

        iter += 1
    os.remove('../../TEMP.phy')
    os.remove('../../TEMP.tre')


if __name__ == "__main__":
    description = 'This removes the fastest evolving sites within the phylogenomic supermatrix in a stepwise fashion.'
    parser, optional, required = help_formatter.initialize_argparse(name='fast_site_remover.py',
                                                                    desc=description,
                                                                    usage='fast_site_remover.py '
                                                                          '[OPTIONS] -i /path/to/input/')

    # Required Arguments
    required.add_argument('-m', '--matrix', required=True, type=str, metavar='<matrix>',
                          help=textwrap.dedent("""\
                              Path to matrix
                              """))
    required.add_argument('-tr', '--tree', type=str, metavar='',
                          help=textwrap.dedent("""\
                                Path to tree
                                  """))

    # Optional Arguments
    optional.add_argument('-s', '--step_size', type=int, default=3000, metavar='N',
                          help=textwrap.dedent("""\
                                Size of removal step (i.e., 1000 sites removed) to exhaustion
                                Default: 3000
                                """))
    optional.add_argument('-if', '--in_format', metavar='<format>', type=str, default='fasta',
                          help=textwrap.dedent("""\
                                Format of input matrix.
                                Options: fasta, nexus, phylip (names truncated at 10 characters), 
                                or phylip-relaxed (names are not truncated)
                                Default: fasta
                                """))
    optional.add_argument('-of', '--out_format', metavar='<format>', type=str, default='fasta',
                          help=textwrap.dedent("""\
                                Desired format of the output steps.
                                Options: fasta, nexus, phylip (names truncated at 10 characters), 
                                or phylip-relaxed (names are not truncated)
                                Default: fasta
                                """))

    args = help_formatter.get_args(parser, optional, required, pre_suf=False, inp_dir=False)

    os.mkdir(args.output)

    main()
